#!/bin/sh
# Tar installer object
#
# This file is saved on the disk when the .tar package is installed by
# vmware-install.pl. It can be invoked by any installer.

#
# Tools
#

# BEGINNING_OF_TMPDIR_DOT_SH
#!/bin/sh

# Create a temporary directory
#
# They are a lot of small utility programs to create temporary files in a
# secure way, but none of them is standard. So I wrote this --hpreg
make_tmp_dir() {
  local dirname="$1" # OUT
  local prefix="$2"  # IN
  local tmp
  local serial
  local loop

  tmp="${TMPDIR:-/tmp}"

  # Don't overwrite existing user data
  # -> Create a directory with a name that didn't exist before
  #
  # This may never succeed (if we are racing with a malicious process), but at
  # least it is secure
  serial=0
  loop='yes'
  while [ "$loop" = 'yes' ]; do
    # Check the validity of the temporary directory. We do this in the loop
    # because it can change over time
    if [ ! -d "$tmp" ]; then
      echo 'Error: "'"$tmp"'" is not a directory.'
      echo
      exit 1
    fi
    if [ ! -w "$tmp" -o ! -x "$tmp" ]; then
      echo 'Error: "'"$tmp"'" should be writable and executable.'
      echo
      exit 1
    fi

    # Be secure
    # -> Don't give write access to other users (so that they can not use this
    # directory to launch a symlink attack)
    if mkdir -m 0755 "$tmp"'/'"$prefix$serial" >/dev/null 2>&1; then
      loop='no'
    else
      serial=`expr "$serial" + 1`
      if [ `expr "$serial" % 200` = '0' ]; then
        echo 'Warning: The "'"$tmp"'" directory may be under attack.'
        echo
      fi
    fi
  done

  eval "$dirname"'="$tmp"'"'"'/'"'"'"$prefix$serial"'
}

# END_OF_TMPDIR_DOT_SH

# BEGINNING_OF_DB_DOT_SH
#!/bin/sh

#
# Manage an installer database
#

# Load an answer from a database on disk
db_load_answer() {
  local dbfile="$1" # IN
  local id="$2"     # IN

  grep '^remove_answer '"$id"'$\|^answer '"$id"' ' "$dbfile" | tail -1 | sed -n 's/^answer '"$id"' //p'
}

# Load all answers from a database on disk to memory (<dbvar>_answer_*
# variables)
db_load() {
  local dbvar="$1"  # OUT
  local dbfile="$2" # IN
  local value
  local id
  local answers
  local answers2

  answers=''
  if [ -r "$dbfile" ]; then
    for id in `sed -n -e 's/^answer \(.*\) .*/\1/p' "$dbfile"`; do
      answers2=`echo "$answers" | sed 's/ '"$id"' //'`
      if [ "$answers2" = "$answers" ]; then
        value=`db_load_answer "$dbfile" "$id"`
        if [ "$value" != '' ]; then
          eval "$dbvar"'_answer_'"$id"'="$value"'
          answers="$answers"' '"$id"' '
        fi
      fi
    done
  fi
  eval "$dbvar"'_answers="$answers"'
}

# Iterate through all answers in the database in memory, calling <func> with
# id/value pairs and the remaining arguments to this function
db_iterate() {
  local dbvar="$1" # IN
  local func="$2"  # IN
  shift 2
  local answers
  local i
  local value

  eval 'answers="$'"$dbvar"'_answers"'
  # There is no double quote around $answers on purpose
  for i in $answers; do
    eval 'value="$'"$dbvar"'_answer_'"$i"'"'
    eval "$func"' "$i" "$value" "$@"'
  done
}

# If it exists in memory, remove an answer from a database (disk and memory)
db_remove_answer() {
  local dbvar="$1"  # IN
  local dbfile="$2" # IN
  local id="$3"     # IN
  local answers
  local answers2

  eval 'answers="$'"$dbvar"'_answers"'
  answers2=`echo "$answers" | sed 's/ '"$id"' //'`
  if [ "$answers2" != "$answers" ]; then
    # The answer is in memory
    eval 'unset '"$dbvar"'_answer_'"$id"
    eval "$dbvar"'_answers="$answers2"'
    echo 'remove_answer '"$id" >> "$dbfile"
  fi
}

# Add an answer to a database (disk and memory)
db_add_answer() {
  local dbvar="$1"  # IN
  local dbfile="$2" # IN
  local id="$3"     # IN
  local value="$4"  # IN

  db_remove_answer "$dbvar" "$dbfile" "$id"
  eval "$dbvar"'_answer_'"$id"'="$value"'
  eval "$dbvar"'_answers="$'"$dbvar"'_answers"'"'"' '"'"'"$id"'"'"' '"'"
  echo 'answer '"$id"' '"$value" >> "$dbfile"
}

# Add a file to a database on disk
# 'file' is the file to put in the database (it may not exist on the disk)
# 'tsfile' is the file to get the timestamp from, '' if no timestamp
db_add_file() {
  local dbfile="$1"
  local file="$2"
  local tsfile="$3"
  local date

  if [ "$tsfile" = '' ]; then
    echo 'file '"$file" >> "$dbfile"
  else
    date=`date -r "$tsfile" '+%s' 2> /dev/null`
    if [ ! "$date" = '' ]; then
      date=' '"$date"
    fi
    echo 'file '"$file$date" >> "$dbfile"
  fi
}

# Add a directory to a database on disk
db_add_dir() {
  local dbfile="$1"
  local dir="$2"

  echo 'directory '"$dir" >> "$dbfile"
}

# END_OF_DB_DOT_SH

#
# Implementation of the methods
#

# Return the human-readable type of the installer
installer_kind() {
  echo 'tar'

  exit 0
}

# Return the human-readable version of the installer
installer_version() {
  echo '2'

  exit 0
}

# Convert the installer database format to formats used by older installers
# The output should be a .tar.gz containing enough information to allow a
# clean "upgrade" (which will actually be a downgrade) by an older installer
installer_convertdb() {
  local format="$1"
  local output="$2"
  local tmpdir

  case "$format" in
    tar2|rpm2)
      if [ "$format" = 'tar2' ]; then
        echo 'Keeping the tar2 installer database format.'
      else
        echo 'Converting the tar2 installer database format'
        echo '        to the rpm2 installer database format.'
      fi
      echo
      # The next installer uses the same database format. Backup a full
      # database state that it can use as a fresh new database.
      #
      # Everything should go in:
      #  /etc/vmware/
      #              state/
      #
      # But those directories do not have to be referenced,
      # the next installer will do it just after restoring the backup
      # because only it knows if those directories have been created.
      #
      # Also, do not include those directories in the backup, because some
      # versions of tar (1.13.17+ are ok) do not untar directory permissions
      # as described in their documentation.
      make_tmp_dir 'tmpdir' 'vmware-installer'
      mkdir -p "$tmpdir"'/etc/vmware'
      db_add_file "$tmpdir"'/etc/vmware/locations' '/etc/vmware/locations' ''
      db_load 'db' '/etc/vmware/locations'
      write() {
        local id="$1"
        local value="$2"
        local dbfile="$3"

        echo 'answer '"$id"' '"$value" >> "$dbfile"
      }
      db_iterate 'db' 'write' "$tmpdir"'/etc/vmware/locations'
      files='./etc/vmware/locations'
      if [ "$db_answer_VNET_SAMBA" = 'yes' ]; then
        mkdir -p "$tmpdir"'/etc/vmware/state'
        cp /etc/vmware/vmnet1/smb/private/smbpasswd "$tmpdir"'/etc/vmware/state/1'
        db_add_file "$tmpdir"'/etc/vmware/locations' '/etc/vmware/state/1' "$tmpdir"'/etc/vmware/state/1'
        echo 'answer VNET_SAMBA_SMBPASSWD /etc/vmware/state/1' >> "$tmpdir"'/etc/vmware/locations'
        files="$files"' ./etc/vmware/state/1'
        cp /etc/vmware/vmnet1/smb/private/MACHINE.SID "$tmpdir"'/etc/vmware/state/2'
        db_add_file "$tmpdir"'/etc/vmware/locations' '/etc/vmware/state/2' "$tmpdir"'/etc/vmware/state/2'
        echo 'answer VNET_SAMBA_MACHINESID /etc/vmware/state/2' >> "$tmpdir"'/etc/vmware/locations'
        files="$files"' ./etc/vmware/state/2'
      fi
      # There is no double quote around $files on purpose
      tar -C "$tmpdir" -czopf "$output" $files 2> /dev/null
      rm -rf "$tmpdir"

      exit 0
      ;;

    tar|rpm)
      echo 'Converting the tar2 installer database format'
      echo '        to the '"$format"'  installer database format.'
      echo
      # Backup only the main database file. The next installer ignores
      # new keywords as well as file and directory statements, and deals
      # properly with remove_ statements
      tar -C '/' -czopf "$output" './etc/vmware/locations' 2> /dev/null

      exit 0
      ;;

    old)
      echo 'Converting the tar2 installer database format'
      echo '        to the old  installer database format.'
      echo
      # Backup only the main database file (there is no need to reference
      # it). Remove all file and directory statements because the
      # uninstallation will be handled by the uninstall method in this
      # file. The next installer ignore other keywords. Unfortunately,
      # answer statements are read (sometimes wrongly, because the next
      # installer doesn't know about remove_ statements) before this is
      # executed. Basically, we can only help to make the old uninstaller
      # look good here.
      make_tmp_dir 'tmpdir' 'vmware-installer'
      mkdir -p "$tmpdir"'/etc/vmware'
      echo 'VERSION 2.0.3' >> "$tmpdir"'/etc/vmware/locations'
      tar -C "$tmpdir" -czopf "$output" './etc/vmware/locations' 2> /dev/null
      rm -rf "$tmpdir"

      exit 0
      ;;

    *)
      echo 'Unknown '"$format"' installer database format.'
      echo

      exit 1
      ;;
  esac
}

# Uninstall what has been installed by the installer.
# This should never prompt the user, because it can be annoying if invoked
# from the rpm installer for example.
installer_uninstall() {
  bindir=`db_load_answer '/etc/vmware/locations' 'BINDIR'`
  if [ "$bindir" = '' ]; then
    echo 'Error: Unable to find the binary installation directory (answer BINDIR)'
    echo '       in the installer database file "/etc/vmware/locations".'
    echo

    exit 1
  fi

  # Remove the package
  if [ ! -x "$bindir"'/vmware-uninstall.pl' ]; then
    echo 'Error: Unable to execute "'"$bindir"'/vmware-uninstall.pl".'
    echo

    exit 1
  fi
  "$bindir"/vmware-uninstall.pl || exit 1

  exit 0
}

#
# Interface of the methods
#

case "$1" in
  kind)
    installer_kind
    ;;

  version)
    installer_version
    ;;

  convertdb)
    installer_convertdb "$2" "$3"
    ;;

  uninstall)
    installer_uninstall
    ;;

  *)
    echo 'Usage: '`basename "$0"`' {kind|version|convertdb|uninstall}'
    echo

    exit 1
    ;;
esac

